(*pp camlp4o *)

open Global

let open_connection host port =
  let he = Unix.gethostbyname host in
  let addr = he.Unix.h_addr_list.(0) in
  Unix.open_connection (Unix.ADDR_INET (addr, port))

let output = ref(stdout)
let input = ref(stdin)

let print_player () = 
  let output = !output in
  output_string output "Player\n";
  flush output

let make_cnx () =
  let (ninput, noutput) = open_connection Sys.argv.(1)
			  (int_of_string Sys.argv.(2)) in
  output := noutput;
  input := ninput
  
let read_game () =
  let input = !input in
  let l1 = input_line input in
  let (sx, sy) = Scanf.sscanf l1 "%d %d" (fun x y -> (x, y)) in
  Printf.printf "%d %d\n" sx sy;
  for y = 1 to sy do
    let li = input_line input in
    for x = 1 to sx do
      match li.[x - 1] with
      | '.' -> game.(x).(y) <- Good
      | '~' -> game.(x).(y) <- Bad
      | '#' -> game.(x).(y) <- Bad
      | '@' -> game.(x).(y) <- Good
      | _ -> assert false
    done
  done
  
let read_initial_stat () =
  let input = !input in
  let li = input_line input in
  let id, car = Scanf.sscanf li "%d %d" (fun x y -> (x, y)) in
  { rid = id; position_x = 0; position_y = 0;
    carrying = {
      max_weight = car;
      carryng_weight = 0;
      carrying_package = []
    }
  }

open Genlex
let lexer = make_lexer []

let read_info r =
  let rec read_info r = parser
    | [< 'Ident "#"; rest >] ->
	begin
	  match rest with parser
	    | [< 'Int i when i = r.rid; rest >] -> 
		read_my_info r rest
	    | [< 'x; rest >] ->
		read_info r rest 
	end
    | [< 'x; rest >] -> 
	read_info r rest 
    | [< >] -> assert false
  and read_my_info r = parser
    | [< 'Ident "X"; 'Int i; 'Ident "Y"; 'Int j; rest >] ->
	read_my_info { r with position_x = i; position_y = j } rest
    | [< 'Ident "N"; rest >] ->
	read_my_info { r with position_y = r.position_y + 1 } rest
    | [< 'Ident "S"; rest >] ->
	read_my_info { r with position_y = r.position_y - 1 } rest
    | [< 'Ident "E"; rest >] ->
	read_my_info { r with position_x = r.position_x + 1 } rest
    | [< 'Ident "W"; rest >] ->
	read_my_info { r with position_x = r.position_x - 1 } rest
    | [< 'Ident "P"; 'Int i; rest >] ->
	let p = package.(i) in
	assert (p <> bad_package);
	package.(i) <- bad_package;
	let nc = r.carrying in
	let nc = { nc with carrying_package = p::nc.carrying_package;
		     carryng_weight = p.weight + nc.carryng_weight } in
	read_my_info { r with carrying = nc } rest
    | [< 'Ident "D"; 'Int i; rest >] ->
	let nc = r.carrying in
	let lp = nc.carrying_package in
	let rec aux i l =
	  match l with
	  | [] -> assert false
	  | x::rest when x.pid = i -> (x,rest)
	  | x::rest ->
	      let (res, left) = aux i rest in
	      (res,x::left) in
	let p, lp = aux i lp in
	let nc = { nc with carrying_package = lp;
		     carryng_weight = nc.carryng_weight - p.weight } in
	read_my_info { r with carrying = nc } rest
    | [< >] -> r in
  let li = input_line !input in
  let s = Stream.of_string li in
  let s = lexer s in
  read_info r s
    
let read_available () =
  let rec read_available accu = parser
    | [< 'Int id; 'Int to_x; 'Int to_y; 'Int w; rest >] ->
	let p = { pid = id; weight = w; going_x = to_x; going_y = to_y } in
	package.(id) <- p;
	read_available (p::accu) rest
    | [< >] -> accu in
  let li = input_line !input in
  if li <> "" then 
    begin 
      print_string li;
      print_newline ()
    end;
  let s = Stream.of_string li in
  let s = lexer s in
  read_available [] s
  

let write_order o =
  let output = !output in
  output_string output "1 ";
  print_string "1 ";
  begin
    match o with
    | Pick l ->
	output_string output "Pick";
	print_string "Pick";
	List.iter (fun x -> output_char output ' ';
		     output_string output (string_of_int x)) l;
	List.iter (fun x -> print_char ' ';
		     print_string (string_of_int x)) l
    | Drop l ->
	output_string output "Drop";
	print_string "Drop";
	List.iter (fun x -> output_char output ' ';
		     output_string output (string_of_int x)) l;
	List.iter (fun x -> print_char ' ';
		     print_string (string_of_int x)) l
    | Move N ->
	output_string output "Move N";
	print_string "Move N"
    | Move S -> 
	output_string output "Move S";
	print_string "Move S"
    | Move E -> 
	output_string output "Move E";
	print_string "Move E"
    | Move W -> 
	output_string output "Move W";
	print_string "Move W"
  end;
  output_string output "\n";
  print_string "\n";
  flush output;
  flush stdout
	
